'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/**
 * Find the first autolink.json file, starting from process.cwd()
 */
let getFirstNpmAutoLinkFile = (() => {
  var _ref = _asyncToGenerator(function* () {
    let currentDir = process.cwd();
    let foundPath = null;
    let oldDir = null;

    var promises = [];
    do {
      const autoLinkPath = path.join(currentDir, AUTOLINK_FILE_NAME);
      const fileExists = yield fs.pathExists(autoLinkPath);
      if (fileExists) {
        return autoLinkPath;
      }
      oldDir = currentDir;
      currentDir = path.dirname(currentDir);
    } while (currentDir !== oldDir);

    throw new Error('Autolink file not found');
  });

  return function getFirstNpmAutoLinkFile() {
    return _ref.apply(this, arguments);
  };
})();

let link = (() => {
  var _ref2 = _asyncToGenerator(function* () {
    const filePath = yield getFirstNpmAutoLinkFile();
    const fileContent = getAutolinkContent(filePath);
    if (!fileContent.link) {
      console.log('Nothing to link');
      return;
    }

    const workspaceDir = path.dirname(filePath);
    const packages = yield getPackagesFromPath(workspaceDir);
    const dependencyTree = yield getTree(packages, workspaceDir);

    const toLink = _.reduce(dependencyTree, function (res, projectConf, projectName) {
      const linkList = fileContent.link[projectName];
      if (linkList && projectConf.dependencies) {
        // If we have a linkList for this project && it has dependencies
        const toLinkInProject = _.reduce(projectConf.dependencies, function (res2, depConf, depName) {
          fileContent.link;
          if (linkList.includes(depName)) {
            // If dependency is in linkList
            const sourceConf = dependencyTree[depName];
            res2.push({
              source: {
                path: path.join(workspaceDir, sourceConf.path),
                name: depName,
                bin: sourceConf.bin
              },
              target: {
                path: path.join(workspaceDir, projectConf.path)
              }
            });
          }
          return res2;
        }, []);
        res.push(...toLinkInProject);
      }
      return res;
    }, []);

    for (let linkConf of toLink) {
      yield linkModule(linkConf);
    }

    return ls();
  });

  return function link() {
    return _ref2.apply(this, arguments);
  };
})();

let unlink = (() => {
  var _ref3 = _asyncToGenerator(function* () {
    const filePath = yield getFirstNpmAutoLinkFile();
    const workspaceDir = path.dirname(filePath);
    const packages = yield getPackagesFromPath(workspaceDir);

    for (let projectName in packages) {
      const projectConf = packages[projectName];
      const nodeModulesPath = path.join(workspaceDir, projectConf.path, 'node_modules');
      yield removeLinksInDir(nodeModulesPath);
    }

    return ls();
  });

  return function unlink() {
    return _ref3.apply(this, arguments);
  };
})();

let ls = (() => {
  var _ref4 = _asyncToGenerator(function* () {
    const filePath = yield getFirstNpmAutoLinkFile();
    const workspaceDir = path.dirname(filePath);
    const packages = yield getPackagesFromPath(workspaceDir);
    const dependencyTree = yield getTree(packages, workspaceDir);
    return getWorkspaceDisplayTree(dependencyTree);
  });

  return function ls() {
    return _ref4.apply(this, arguments);
  };
})();

let getTree = (() => {
  var _ref5 = _asyncToGenerator(function* (packages, workspaceDir) {
    const dependencyTree = _.reduce(packages, function (res, conf) {
      const pack = conf.package;
      res[pack.name] = {
        path: conf.path,
        bin: pack.bin
      };
      res[pack.name].dependencies = _.reduce(_extends({}, pack.dependencies, pack.devDependencies), function (res, depVersion, depName) {
        if (Object.keys(packages).includes(depName)) {
          res[depName] = {
            semver: depVersion
          };
        }
        return res;
      }, {});
      return res;
    }, {});

    for (let packName in dependencyTree) {
      const conf = dependencyTree[packName];
      const nodeModulesDir = path.join(workspaceDir, conf.path, 'node_modules');
      for (let depName in conf.dependencies) {
        const depInfo = conf.dependencies[depName];
        Object.assign(depInfo, (yield getPackageStatus(nodeModulesDir, depName, workspaceDir)));
      }
    }
    return dependencyTree;
  });

  return function getTree(_x, _x2) {
    return _ref5.apply(this, arguments);
  };
})();

let getPackagesFromPath = (() => {
  var _ref6 = _asyncToGenerator(function* (workspaceDir) {
    const files = yield glob('**/package.json', {
      cwd: workspaceDir,
      ignore: ['**/node_modules/**', '**/bower_components/**']
    });

    return files.reduce(function (res, file) {
      const fileAbsolutePath = path.join(workspaceDir, file);
      const pack = require(fileAbsolutePath);
      res[pack.name] = {
        package: pack,
        path: path.dirname(file)
      };
      return res;
    }, {});
  });

  return function getPackagesFromPath(_x3) {
    return _ref6.apply(this, arguments);
  };
})();

/**
 * source:
 *  name
 *  path
 *  bin
 * target:
 *  path
 *
 * @param {*} linkConf
 */


let linkModule = (() => {
  var _ref7 = _asyncToGenerator(function* (linkConf) {
    var scopeMatch = linkConf.source.name.match(/^(@.*)\/.*/);
    var scope;
    if (!!scopeMatch) {
      scope = scopeMatch[1];
    }
    const nodeModulesPath = path.join(linkConf.target.path, 'node_modules');
    var scopedPath = scope ? path.join(nodeModulesPath, scope) : nodeModulesPath;
    var targetPath = path.join(nodeModulesPath, linkConf.source.name);
    var binPath = path.join(nodeModulesPath, '.bin');
    var sourcePath = linkConf.source.path;

    var backPath = getBackupPath(targetPath);

    let stat;
    try {
      stat = yield fs.lstat(targetPath);
    } catch (e) {
      console.log('package not installed');
    }

    if (stat) {
      if (stat.isSymbolicLink()) {
        //If symlink already exist then remove it
        yield fs.remove(targetPath);
      } else {
        //if real directory, remove backPath and rename
        yield fs.remove(backPath);
        yield fs.rename(targetPath, backPath);
      }
    }

    yield fs.ensureDir(scopedPath);
    yield fs.symlink(sourcePath, targetPath, 'junction');
    yield fs.ensureDir(binPath);
    yield linkBins(binPath, linkConf.source.bin, sourcePath);
  });

  return function linkModule(_x4) {
    return _ref7.apply(this, arguments);
  };
})();

let removeLinksInDir = (() => {
  var _ref8 = _asyncToGenerator(function* (dir) {
    const links = yield listLinksInDir(dir);

    for (let linkPath of links) {
      yield fs.remove(linkPath);
      try {
        yield fs.rename(getBackupPath(linkPath), linkPath);
      } catch (e) {
        // console.error(e);
      }
    }
  });

  return function removeLinksInDir(_x5) {
    return _ref8.apply(this, arguments);
  };
})();

let getPackageStatus = (() => {
  var _ref9 = _asyncToGenerator(function* (nodeModulesDir, packageName, workspaceDir) {
    const moduleDir = path.join(nodeModulesDir, packageName);
    try {
      const stat = yield fs.lstat(moduleDir);
      const pack = require(path.join(moduleDir, 'package.json'));
      const res = {
        name: pack.name,
        version: pack.version
      };

      if (stat.isSymbolicLink()) {
        const linkTarget = yield fs.readlink(moduleDir);
        res.isLink = true;
        res.linkTarget = linkTarget;
        if (linkTarget.startsWith(workspaceDir)) {
          res.linkTarget = linkTarget.substr(workspaceDir.length);
        }
      }
      return res;
    } catch (e) {
      return null;
    }
  });

  return function getPackageStatus(_x6, _x7, _x8) {
    return _ref9.apply(this, arguments);
  };
})();

let listLinksInDir = (() => {
  var _ref10 = _asyncToGenerator(function* (directory) {
    yield fs.ensureDir(directory);
    const files = yield fs.readdir(directory);

    const links = [];

    for (let file of files) {
      var filePath = path.join(directory, file);
      if (file[0] === '@') {
        const scopedLinks = yield listLinksInDir(filePath);
        links.push(...scopedLinks);
        continue;
      }

      const stat = yield fs.lstat(filePath);
      if (stat.isSymbolicLink()) {
        links.push(filePath);
      }
      // return fs.readlinkAsync(file);)
    }

    return links;
  });

  return function listLinksInDir(_x9) {
    return _ref10.apply(this, arguments);
  };
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var Promise = require('bluebird');
Promise.longStackTraces();
var fs = require('fs-extra');
var path = require('path');
var glob = Promise.promisify(require('glob'));
var endOfLine = require('os').EOL;
var _ = require('lodash');
var pathIsAbsolute = require('path-is-absolute');
var semver = require('semver');
var chalk = require('chalk');

const AUTOLINK_FILE_NAME = 'autolink.json';

function getNodeModulesPath() {
  return path.join(process.cwd(), 'node_modules');
}

function getAutolinkContent(filePath) {
  return require(filePath);
}

function getWorkspaceDisplayTree(dependencyTree) {
  return _.reduce(dependencyTree, (res, projectConf, projectName) => {
    const obj = _.reduce(projectConf.dependencies, (res2, depConf, depName) => {
      res2[depName] = `semver: ${depConf.semver}`;
      if (!depConf.version) {
        res2[depName] += ', not found';
      } else {
        if (depConf.isLink) {
          res2[depName] += `, linked: ${depConf.version} (${depConf.linkTarget})`;
        } else {
          res2[depName] += `, installed: ${depConf.version}`;
        }
      }
      return res2;
    }, {});
    const key = projectName === projectConf.path ? projectName : `${projectName} (${projectConf.path})`;
    res[key] = obj;
    return res;
  }, {});
}

function linkBins(binPath, bins, sourcePath) {
  return Promise.all(_.reduce(bins, function (res, bin, binName) {
    var sourceBin = path.join(sourcePath, bin);
    var targetLink = path.join(binPath, binName);
    //var bakTargetLink = path.join(binPath, binName) + '.bak';

    var promise = fs.remove(targetLink).then(function () {
      return fs.symlink(sourceBin, targetLink, 'junction');
    }).then(function () {
      return fs.chmod(targetLink, '755');
    });

    res.push(promise);
    return res;
  }, []));
}

function getBackupPath(packagePath) {
  const depName = path.basename(packagePath);
  return path.join(path.dirname(packagePath), `.${depName}.autolinkbackup`);
}

module.exports = {
  link,
  unlink,
  ls
};